@@ -50,8 +50,9 @@ static void help(void);
 static void setcom(int);
 int decide_tag_by_context(const char *, const char *, int);
 int main(int, char **);
-void completion(const char *, const char *, const char *);
+void completion(const char *, const char *, const char *, int);
 void completion_idutils(const char *, const char *, const char *);
+void completion_path(const char *, const char *);
 void idutils(const char *, const char *);
 void grep(const char *, char *const *, const char *);
 void pathlist(const char *, const char *);
@@ -94,9 +95,8 @@ int literal;				/* 1: literal search	*/
 int print0;				/* -print0 option	*/
 int format;
 int type;				/* path conversion type */
-char cwd[MAXPATHLEN];			/* current directory	*/
-char root[MAXPATHLEN];			/* root of source tree	*/
-char dbpath[MAXPATHLEN];		/* dbpath directory	*/
+int match_part;				/* match part only	*/
+const char *cwd, *root, *dbpath;
 char *context_file;
 char *context_lineno;
 char *file_list;
@@ -119,9 +119,13 @@ help(void)
 #define RESULT		128
 #define FROM_HERE	129
 #define ENCODE_PATH	130
+#define MATCH_PART	131
 #define SORT_FILTER     1
 #define PATH_FILTER     2
 #define BOTH_FILTER     (SORT_FILTER|PATH_FILTER)
+#define MATCH_PART_FIRST 1
+#define MATCH_PART_LAST  2
+#define MATCH_PART_ALL   3
 
 static struct option const long_options[] = {
 	{"absolute", no_argument, NULL, 'a'},
@@ -156,6 +160,7 @@ static struct option const long_options[] = {
 	{"from-here", required_argument, NULL, FROM_HERE},
 	{"debug", no_argument, &debug, 1},
 	{"literal", no_argument, &literal, 1},
+	{"match-part", required_argument, NULL, MATCH_PART},
 	{"print0", no_argument, &print0, 1},
 	{"version", no_argument, &show_version, 1},
 	{"help", no_argument, &show_help, 1},
@@ -170,9 +175,9 @@ setcom(int c)
 {
 	if (command == 0)
 		command = c;
-	else if (c == 'c' && command == 'I')
+	else if (c == 'c' && (command == 'I' || command == 'P'))
 		command = c;
-	else if (c == 'I' && command == 'c')
+	else if (command == 'c' && (c == 'I' || c == 'P'))
 		;
 	else if (command != c)
 		usage();
@@ -326,7 +331,6 @@ main(int argc, char **argv)
 			break;
 		case 'd':
 			dflag++;
-			setcom(optchar);
 			break;
 		case 'e':
 			av = optarg;
@@ -433,6 +437,16 @@ main(int argc, char **argv)
 			context_file = p;
 			}
 			break;
+		case MATCH_PART:
+			if (!strcmp(optarg, "first"))
+				match_part = MATCH_PART_FIRST;
+			else if (!strcmp(optarg, "last"))
+				match_part = MATCH_PART_LAST;
+			else if (!strcmp(optarg, "all"))
+				match_part = MATCH_PART_ALL;
+			else
+				die_with_code(2, "unknown part type for the --match-part option.");
+			break;
 		case RESULT:
 			if (!strcmp(optarg, "ctags-x"))
 				format = FORMAT_CTAGS_X;
@@ -512,13 +526,15 @@ main(int argc, char **argv)
 		nosource = 1;	/* to keep compatibility */
 	if (print0)
 		set_print0();
+	if (cflag && match_part == 0)
+		match_part = MATCH_PART_ALL;
 	/*
 	 * remove leading blanks.
 	 */
 	if (!Iflag && !gflag && av)
 		for (; *av == ' ' || *av == '\t'; av++)
 			;
-	if (cflag && av && isregex(av))
+	if (cflag && !Pflag && av && isregex(av))
 		die_with_code(2, "only name char is allowed with -c option.");
 	/*
 	 * get path of following directories.
@@ -526,9 +542,16 @@ main(int argc, char **argv)
 	 *	o root of source tree
 	 *	o dbpath directory
 	 *
-	 * if GTAGS not found, getdbpath doesn't return.
+	 * if GTAGS not found, exit with an error message.
 	 */
-	getdbpath(cwd, root, dbpath, (pflag && vflag));
+	{
+		int status = setupdbpath(pflag && vflag);
+		if (status < 0)
+			die_with_code(-status, gtags_dbpath_error);
+		cwd = get_cwd();
+		root = get_root();
+		dbpath = get_dbpath();
+	}
 	/*
 	 * print dbpath or rootdir.
 	 */
@@ -558,39 +581,6 @@ main(int argc, char **argv)
 		strbuf_close(sb);
 		exit(0);
 	}
-	/*
-	 * complete function name
-	 */
-	if (cflag) {
-		if (Iflag)
-			completion_idutils(dbpath, root, av);
-		else
-			completion(dbpath, root, av);
-		exit(0);
-	}
-	/*
-	 * make local prefix.
-	 * local prefix must starts with './' and ends with '/'.
-	 */
-	if (lflag) {
-		STRBUF *sb = strbuf_open(0);
-
-		strbuf_putc(sb, '.');
-		if (strcmp(root, cwd) != 0) {
-			char *p = cwd + strlen(root);
-			if (*p != '/')
-				strbuf_putc(sb, '/');
-			strbuf_puts(sb, p);
-		}
-		strbuf_putc(sb, '/');
-		localprefix = check_strdup(strbuf_value(sb));
-		strbuf_close(sb);
-#ifdef DEBUG
-		fprintf(stderr, "root=%s\n", root);
-		fprintf(stderr, "cwd=%s\n", cwd);
-		fprintf(stderr, "localprefix=%s\n", localprefix);
-#endif
-	}
 	/*
 	 * decide tag type.
 	 */
@@ -606,6 +596,41 @@ main(int argc, char **argv)
 		else
 			db = (rflag) ? GRTAGS : ((sflag) ? GSYMS : GTAGS);
 	}
+	/*
+	 * complete function name
+	 */
+	if (cflag) {
+		if (Iflag)
+			completion_idutils(dbpath, root, av);
+		else if (Pflag)
+			completion_path(dbpath, av);
+		else
+			completion(dbpath, root, av, db);
+		exit(0);
+	}
+	/*
+	 * make local prefix.
+	 * local prefix must starts with './' and ends with '/'.
+	 */
+	if (lflag) {
+		STRBUF *sb = strbuf_open(0);
+
+		strbuf_putc(sb, '.');
+		if (strcmp(root, cwd) != 0) {
+			const char *p = cwd + strlen(root);
+			if (*p != '/')
+				strbuf_putc(sb, '/');
+			strbuf_puts(sb, p);
+		}
+		strbuf_putc(sb, '/');
+		localprefix = check_strdup(strbuf_value(sb));
+		strbuf_close(sb);
+#ifdef DEBUG
+		fprintf(stderr, "root=%s\n", root);
+		fprintf(stderr, "cwd=%s\n", cwd);
+		fprintf(stderr, "localprefix=%s\n", localprefix);
+#endif
+	}
 	/*
 	 * decide format.
 	 * The --result option is given to priority more than the -t and -x option.
@@ -670,12 +695,13 @@ main(int argc, char **argv)
  *	i)	dbpath	dbpath directory
  *	i)	root	root directory
  *	i)	prefix	prefix of primary key
+ *	i)	db	GTAGS,GRTAGS,GSYMS
  */
 void
-completion(const char *dbpath, const char *root, const char *prefix)
+completion(const char *dbpath, const char *root, const char *prefix, int db)
 {
 	int flags = GTOP_KEY;
-	GTOP *gtop = gtags_open(dbpath, root, (sflag) ? GSYMS : GTAGS, GTAGS_READ, 0);
+	GTOP *gtop = gtags_open(dbpath, root, db, GTAGS_READ, 0);
 	GTP *gtp;
 
 	if (prefix && *prefix == 0)	/* In the case global -c '' */
@@ -786,6 +812,57 @@ completion_idutils(const char *dbpath, const char *root, const char *prefix)
 	fclose(ip);
 	strbuf_close(sb);
 }
+/*
+ * completion_path: print candidate path list.
+ *
+ *	i)	dbpath	dbpath directory
+ *	i)	prefix	prefix of primary key
+ */
+void
+completion_path(const char *dbpath, const char *prefix)
+{
+	GFIND *gp;
+	const char *localprefix = "./";
+	FILE *op = popen("sort | uniq", "w");
+	const char *path;
+	int prefix_length;
+	int target = GPATH_SOURCE;
+	int flags = (match_part == MATCH_PART_LAST) ? MATCH_LAST : MATCH_FIRST;
+
+	if (prefix && *prefix == 0)	/* In the case global -c '' */
+		prefix = NULL;
+	prefix_length = (prefix == NULL) ? 0 : strlen(prefix);
+	if (oflag)
+		target = GPATH_BOTH;
+	if (Oflag)
+		target = GPATH_OTHER;
+	if (iflag)
+		flags |= IGNORE_CASE;
+	gp = gfind_open(dbpath, localprefix, target);
+	while ((path = gfind_read(gp)) != NULL) {
+		path += 2;				/* skip './'*/
+		if (prefix == NULL) {
+			fputs(path, op);
+			fputc('\n', op);
+		} else if (match_part == MATCH_PART_ALL) {
+			const char *p = path;
+
+			while ((p = locatestring(p, prefix, flags)) != NULL) {
+				fputs(p, op);
+				fputc('\n', op);
+				p += prefix_length;
+			}
+		} else {
+			const char *p = locatestring(path, prefix, flags);
+			if (p != NULL) {
+				fputs(p, op);
+				fputc('\n', op);
+			}
+		}
+	}
+	gfind_close(gp);
+	pclose(op);
+}
 /*
  * Output filter
  *
