@@ -23,9 +23,9 @@
 
 ;; GLOBAL home page is at: http://www.gnu.org/software/global/
 ;; Author: Tama Communications Corporation
-;; Version: 2.10
+;; Version: 3.0
 ;; Keywords: tools
-;; Required version: GLOBAL 5.9 or later
+;; Required version: GLOBAL 5.9.7 or later
 
 ;; Gtags-mode is implemented as a minor mode so that it can work with any
 ;; other major modes. Gtags-select mode is implemented as a major mode.
@@ -99,6 +99,7 @@
   :group 'gtags
   :type 'boolean)
 
+; This has not been used any longer.
 (defcustom gtags-disable-pushy-mouse-mapping nil
   "*If non-nil, mouse key mapping is disabled."
   :group 'gtags
@@ -232,14 +233,17 @@
   (gtags-completing 'gsyms string predicate code))
 (defun gtags-completing-files (string predicate code)
   (gtags-completing 'files string predicate code))
+(defun gtags-completing-idutils (string predicate code)
+  (gtags-completing 'idutils string predicate code))
 ;; common part of completing-XXXX
 ;;   flag: 'gtags or 'gsyms or 'files
 (defun gtags-completing (flag string predicate code)
   ; The purpose of using the -n option for the -P command is to exclude
   ; dependence on the execution directory.
-  (let ((option (cond ((eq flag 'files) "-Pon")
-                      ((eq flag 'gsyms)  "-cs")
-                      (t                "-c")))
+  (let ((option (cond ((eq flag 'files)   "-cPo")
+                      ((eq flag 'gsyms)   "-cs")
+                      ((eq flag 'idutils) "-cI")
+                      (t                  "-c")))
         (complete-list (make-vector 63 0))
         (prev-buffer (current-buffer)))
     ; build completion list
@@ -251,17 +255,10 @@
     ; The completion for symbols matches only to the head of the symbol. But the completion
     ; for files matches any part of the path.
     ;
-    (if (eq flag 'files)
-        ; extract input string and the following part.
-        (let ((match-string (if (equal "" string) "\./\\(.*\\)" (concat ".*\\(" string ".*\\)"))))
-          (while (not (eobp))
-            (looking-at match-string)
-            (intern (gtags-match-string 1) complete-list)
-            (forward-line)))
-      (while (not (eobp))
-        (looking-at gtags-symbol-regexp)
-        (intern (gtags-match-string 0) complete-list)
-        (forward-line)))
+    (while (not (eobp))
+      (looking-at ".*")
+      (intern (gtags-match-string 0) complete-list)
+      (forward-line))
     (kill-buffer (current-buffer))
     ; recover current buffer
     (set-buffer prev-buffer)
@@ -372,12 +369,29 @@
 (defun gtags-find-with-grep ()
   "Input pattern, search with grep(1) and move to the locations."
   (interactive)
-  (gtags-find-with "g"))
+  (let (tagname prompt input)
+    (setq tagname (gtags-current-token))
+    (if tagname
+        (setq prompt (concat "Find pattern: (default " tagname ") "))
+      (setq prompt "Find pattern: "))
+    (setq input (read-from-minibuffer prompt nil nil nil gtags-history-list))
+    (if (not (equal "" input)) (setq tagname input))
+    (gtags-push-context)
+    (gtags-goto-tag tagname "g")))
 
 (defun gtags-find-with-idutils ()
   "Input pattern, search with idutils(1) and move to the locations."
   (interactive)
-  (gtags-find-with "I"))
+  (let (tagname prompt input)
+    (setq tagname (gtags-current-token))
+    (if tagname
+        (setq prompt (concat "Find token: (default " tagname ") "))
+      (setq prompt "Find token: "))
+    (setq input (completing-read prompt 'gtags-completing-idutils
+                  nil nil nil gtags-history-list))
+    (if (not (equal "" input)) (setq tagname input))
+    (gtags-push-context)
+    (gtags-goto-tag tagname "I")))
 
 (defun gtags-find-file ()
   "Input pattern and move to the top of the file."
@@ -492,19 +506,6 @@
 ;; common function
 ;;
 
-;; find with grep or idutils.
-(defun gtags-find-with (flag)
-  (let (tagname prompt input)
-    (setq tagname (gtags-current-token))
-    (if tagname
-        (setq prompt (concat "Find pattern: (default " tagname ") "))
-      (setq prompt "Find pattern: "))
-    (setq input (completing-read prompt 'gtags-completing-gtags
-                 nil nil nil gtags-history-list))
-    (if (not (equal "" input)) (setq tagname input))
-    (gtags-push-context)
-    (gtags-goto-tag tagname flag)))
-
 ;; goto tag's point
 (defun gtags-goto-tag (tagname flag &optional other-win)
   (let (option context save prefix buffer lines flag-char)
@@ -691,6 +692,7 @@ with no args, if that value is non-nil."
   ; Suggested key mapping
   (if gtags-suggested-key-mapping
       (progn
+        ; Key mapping.
         (define-key gtags-mode-map "\eh" 'gtags-display-browser)
         (define-key gtags-mode-map "\C-]" 'gtags-find-tag-from-here)
         (define-key gtags-mode-map "\C-t" 'gtags-pop-stack)
@@ -701,16 +703,16 @@ with no args, if that value is non-nil."
         (define-key gtags-mode-map "\es" 'gtags-find-symbol)
         (define-key gtags-mode-map "\er" 'gtags-find-rtag)
         (define-key gtags-mode-map "\et" 'gtags-find-tag)
-        (define-key gtags-mode-map "\ev" 'gtags-visit-rootdir))
-      nil)
-  ; Mouse key mapping
-  (if gtags-disable-pushy-mouse-mapping nil
-      (if (not gtags-running-xemacs) nil
-       (define-key gtags-mode-map 'button3 'gtags-pop-stack)
-       (define-key gtags-mode-map 'button2 'gtags-find-tag-by-event))
-      (if gtags-running-xemacs nil
-       (define-key gtags-mode-map [mouse-3] 'gtags-pop-stack)
-       (define-key gtags-mode-map [mouse-2] 'gtags-find-tag-by-event))))
+        (define-key gtags-mode-map "\ev" 'gtags-visit-rootdir)
+        ; Mouse key mapping
+        (if (not gtags-running-xemacs) nil
+            (define-key gtags-mode-map 'button3 'gtags-pop-stack)
+            (define-key gtags-mode-map 'button2 'gtags-find-tag-by-event))
+        (if gtags-running-xemacs nil
+            (define-key gtags-mode-map [mouse-3] 'gtags-pop-stack)
+            (define-key gtags-mode-map [mouse-2] 'gtags-find-tag-by-event)))
+  )
+)
 
 ;; make gtags select-mode
 (defun gtags-select-mode ()
@@ -737,13 +739,16 @@ Turning on Gtags-Select mode calls the value of the variable
   (message "[GTAGS SELECT MODE] %d lines" (count-lines (point-min) (point-max)))
   (run-hooks 'gtags-select-mode-hook)
   ; Mouse key mapping
-  (if gtags-disable-pushy-mouse-mapping nil
-      (if (not gtags-running-xemacs) nil
-          (define-key gtags-select-mode-map 'button3 'gtags-pop-stack)
-          (define-key gtags-select-mode-map 'button2 'gtags-select-tag-by-event))
-      (if gtags-running-xemacs nil
-          (define-key gtags-select-mode-map [mouse-3] 'gtags-pop-stack)
-          (define-key gtags-select-mode-map [mouse-2] 'gtags-select-tag-by-event))))
+  (if gtags-suggested-key-mapping
+      (progn
+        (if (not gtags-running-xemacs) nil
+            (define-key gtags-select-mode-map 'button3 'gtags-pop-stack)
+            (define-key gtags-select-mode-map 'button2 'gtags-select-tag-by-event))
+        (if gtags-running-xemacs nil
+            (define-key gtags-select-mode-map [mouse-3] 'gtags-pop-stack)
+            (define-key gtags-select-mode-map [mouse-2] 'gtags-select-tag-by-event)))
+  )
+)
 
 (provide 'gtags)
 
