@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2011
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -88,6 +88,7 @@ extern int show_version;
 extern int caution;
 extern int auto_completion;
 extern int tree_view;
+extern int fixed_guide;
 extern int dynamic;
 extern int symbol;
 extern int statistics;
@@ -129,6 +130,7 @@ extern int tabs;
 extern int flist_fields;
 extern int full_path;
 extern int map_file;
+extern int filemap_file;
 extern const char *icon_suffix;
 extern const char *icon_spec;
 extern const char *prolog_script;
